@@ -51,7 +51,10 @@ bpf_dump(const struct bpf_program *p, int option)
 	for (i = 0; i < n; ++insn, ++i) {
 #ifdef BDEBUG
 		extern int bids[];
-		printf(bids[i] > 0 ? "[%02d]" : " -- ", bids[i] - 1);
+		if (bids[i] > 0)
+			printf("[%02d]", bids[i] - 1);
+		else
+			printf(" -- ");
 #endif
 		puts(bpf_image(insn, i));
 	}
