@@ -11,8 +11,8 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  * notice, this list of conditions and the following disclaimer in the
  * documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote 
- * products derived from this software without specific prior written 
+ * 3. The name of the author may not be used to endorse or promote
+ * products derived from this software without specific prior written
  * permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -31,7 +31,7 @@
  * By Paolo Abeni <paolo.abeni@email.it>
  *
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -73,7 +73,7 @@ struct pcap_bt {
 	int dev_id;		/* device ID of device we're bound to */
 };
 
-int 
+int
 bt_findalldevs(pcap_if_t **alldevsp, char *err_str)
 {
 	struct hci_dev_list_req *dev_list;
@@ -84,18 +84,18 @@ bt_findalldevs(pcap_if_t **alldevsp, char *err_str)
 	sock  = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (sock < 0)
 	{
-		/* if bluetooth is not supported this this is not fatal*/ 
+		/* if bluetooth is not supported this this is not fatal*/
 		if (errno == EAFNOSUPPORT)
 			return 0;
-		snprintf(err_str, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(err_str, PCAP_ERRBUF_SIZE,
 		    "Can't open raw Bluetooth socket: %s", strerror(errno));
 		return -1;
 	}
 
 	dev_list = malloc(HCI_MAX_DEV * sizeof(*dev_req) + sizeof(*dev_list));
-	if (!dev_list) 
+	if (!dev_list)
 	{
-		snprintf(err_str, PCAP_ERRBUF_SIZE, "Can't allocate %zu bytes for Bluetooth device list",
+		pcap_snprintf(err_str, PCAP_ERRBUF_SIZE, "Can't allocate %zu bytes for Bluetooth device list",
 			HCI_MAX_DEV * sizeof(*dev_req) + sizeof(*dev_list));
 		ret = -1;
 		goto done;
@@ -103,9 +103,9 @@ bt_findalldevs(pcap_if_t **alldevsp, char *err_str)
 
 	dev_list->dev_num = HCI_MAX_DEV;
 
-	if (ioctl(sock, HCIGETDEVLIST, (void *) dev_list) < 0) 
+	if (ioctl(sock, HCIGETDEVLIST, (void *) dev_list) < 0)
 	{
-		snprintf(err_str, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(err_str, PCAP_ERRBUF_SIZE,
 		    "Can't get Bluetooth device list via ioctl: %s",
 		    strerror(errno));
 		ret = -1;
@@ -115,11 +115,11 @@ bt_findalldevs(pcap_if_t **alldevsp, char *err_str)
 	dev_req = dev_list->dev_req;
 	for (i = 0; i < dev_list->dev_num; i++, dev_req++) {
 		char dev_name[20], dev_descr[30];
-		
-		snprintf(dev_name, 20, BT_IFACE"%d", dev_req->dev_id);
-		snprintf(dev_descr, 30, "Bluetooth adapter number %d", i);
-			
-		if (pcap_add_if(alldevsp, dev_name, 0, 
+
+		pcap_snprintf(dev_name, 20, BT_IFACE"%d", dev_req->dev_id);
+		pcap_snprintf(dev_descr, 30, "Bluetooth adapter number %d", i);
+
+		if (pcap_add_if(alldevsp, dev_name, 0,
 		       dev_descr, err_str) < 0)
 		{
 			ret = -1;
@@ -171,7 +171,7 @@ bt_create(const char *device, char *ebuf, int *is_ours)
 	/* OK, it's probably ours. */
 	*is_ours = 1;
 
-	p = pcap_create_common(device, ebuf, sizeof (struct pcap_bt));
+	p = pcap_create_common(ebuf, sizeof (struct pcap_bt));
 	if (p == NULL)
 		return (NULL);
 
@@ -190,17 +190,16 @@ bt_activate(pcap_t* handle)
 	int err = PCAP_ERROR;
 
 	/* get bt interface id */
-	if (sscanf(handle->opt.source, BT_IFACE"%d", &dev_id) != 1)
+	if (sscanf(handle->opt.device, BT_IFACE"%d", &dev_id) != 1)
 	{
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
-			"Can't get Bluetooth device index from %s", 
-			 handle->opt.source);
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+			"Can't get Bluetooth device index from %s",
+			 handle->opt.device);
 		return PCAP_ERROR;
 	}
 
 	/* Initialize some components of the pcap structure. */
-	handle->bufsize = handle->snapshot+BT_CTRL_SIZE+sizeof(pcap_bluetooth_h4_header);
-	handle->offset = BT_CTRL_SIZE;
+	handle->bufsize = BT_CTRL_SIZE+sizeof(pcap_bluetooth_h4_header)+handle->snapshot;
 	handle->linktype = DLT_BLUETOOTH_HCI_H4_WITH_PHDR;
 
 	handle->read_op = bt_read_linux;
@@ -212,43 +211,43 @@ bt_activate(pcap_t* handle)
 	handle->setnonblock_op = pcap_setnonblock_fd;
 	handle->stats_op = bt_stats_linux;
 	handlep->dev_id = dev_id;
-	
+
 	/* Create HCI socket */
 	handle->fd = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI);
 	if (handle->fd < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't create raw socket: %s", strerror(errno));
 		return PCAP_ERROR;
 	}
 
 	handle->buffer = malloc(handle->bufsize);
 	if (!handle->buffer) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't allocate dump buffer: %s",
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "Can't allocate dump buffer: %s",
 			pcap_strerror(errno));
 		goto close_fail;
 	}
 
 	opt = 1;
 	if (setsockopt(handle->fd, SOL_HCI, HCI_DATA_DIR, &opt, sizeof(opt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't enable data direction info: %s", strerror(errno));
 		goto close_fail;
 	}
 
 	opt = 1;
 	if (setsockopt(handle->fd, SOL_HCI, HCI_TIME_STAMP, &opt, sizeof(opt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't enable time stamp: %s", strerror(errno));
 		goto close_fail;
 	}
 
-	/* Setup filter, do not call hci function to avoid dependence on 
+	/* Setup filter, do not call hci function to avoid dependence on
 	 * external libs	*/
 	memset(&flt, 0, sizeof(flt));
-	memset((void *) &flt.type_mask, 0xff, sizeof(flt.type_mask));	
+	memset((void *) &flt.type_mask, 0xff, sizeof(flt.type_mask));
 	memset((void *) &flt.event_mask, 0xff, sizeof(flt.event_mask));
 	if (setsockopt(handle->fd, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't set filter: %s", strerror(errno));
 		goto close_fail;
 	}
@@ -261,7 +260,7 @@ bt_activate(pcap_t* handle)
 	addr.hci_channel = HCI_CHANNEL_RAW;
 #endif
 	if (bind(handle->fd, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't attach to device %d: %s", handlep->dev_id,
 		    strerror(errno));
 		goto close_fail;
@@ -282,7 +281,7 @@ bt_activate(pcap_t* handle)
 		if (setsockopt(handle->fd, SOL_SOCKET, SO_RCVBUF,
 		    &handle->opt.buffer_size,
 		    sizeof(handle->opt.buffer_size)) == -1) {
-			snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+			pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 				 "SO_RCVBUF: %s", pcap_strerror(errno));
 			goto close_fail;
 		}
@@ -305,16 +304,19 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 	ssize_t ret;
 	struct pcap_pkthdr pkth;
 	pcap_bluetooth_h4_header* bthdr;
+	u_char *pktd;
+	int in = 0;
 
-	bthdr = (pcap_bluetooth_h4_header*) &handle->buffer[handle->offset];
-	iv.iov_base = &handle->buffer[handle->offset+sizeof(pcap_bluetooth_h4_header)];
+	pktd = (u_char *)handle->buffer + BT_CTRL_SIZE;
+	bthdr = (pcap_bluetooth_h4_header*)(void *)pktd;
+	iv.iov_base = pktd + sizeof(pcap_bluetooth_h4_header);
 	iv.iov_len  = handle->snapshot;
-	
+
 	memset(&msg, 0, sizeof(msg));
 	msg.msg_iov = &iv;
 	msg.msg_iovlen = 1;
 	msg.msg_control = handle->buffer;
-	msg.msg_controllen = handle->offset;
+	msg.msg_controllen = BT_CTRL_SIZE;
 
 	/* ignore interrupt system call error */
 	do {
@@ -327,16 +329,15 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 	} while ((ret == -1) && (errno == EINTR));
 
 	if (ret < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't receive packet: %s", strerror(errno));
 		return -1;
 	}
 
 	pkth.caplen = ret;
 
-	/* get direction and timestamp*/ 
+	/* get direction and timestamp*/
 	cmsg = CMSG_FIRSTHDR(&msg);
-	int in=0;
 	while (cmsg) {
 		switch (cmsg->cmsg_type) {
 			case HCI_CMSG_DIR:
@@ -349,7 +350,7 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 		}
 		cmsg = CMSG_NXTHDR(&msg, cmsg);
 	}
-	if ((in && (handle->direction == PCAP_D_OUT)) || 
+	if ((in && (handle->direction == PCAP_D_OUT)) ||
 				((!in) && (handle->direction == PCAP_D_IN)))
 		return 0;
 
@@ -357,9 +358,8 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 	pkth.caplen+=sizeof(pcap_bluetooth_h4_header);
 	pkth.len = pkth.caplen;
 	if (handle->fcode.bf_insns == NULL ||
-	    bpf_filter(handle->fcode.bf_insns, &handle->buffer[handle->offset],
-	      pkth.len, pkth.caplen)) {
-		callback(user, &pkth, &handle->buffer[handle->offset]);
+	    bpf_filter(handle->fcode.bf_insns, pktd, pkth.len, pkth.caplen)) {
+		callback(user, &pkth, pktd);
 		return 1;
 	}
 	return 0;	/* didn't pass filter */
@@ -368,13 +368,13 @@ bt_read_linux(pcap_t *handle, int max_packets, pcap_handler callback, u_char *us
 static int
 bt_inject_linux(pcap_t *handle, const void *buf, size_t size)
 {
-	snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "inject not supported on "
+	pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE, "inject not supported on "
     		"bluetooth devices");
 	return (-1);
-}                           
+}
 
 
-static int 
+static int
 bt_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 {
 	struct pcap_bt *handlep = handle->priv;
@@ -382,28 +382,28 @@ bt_stats_linux(pcap_t *handle, struct pcap_stat *stats)
 	struct hci_dev_info dev_info;
 	struct hci_dev_stats * s = &dev_info.stat;
 	dev_info.dev_id = handlep->dev_id;
-	
+
 	/* ignore eintr */
 	do {
 		ret = ioctl(handle->fd, HCIGETDEVINFO, (void *)&dev_info);
 	} while ((ret == -1) && (errno == EINTR));
-	    
+
 	if (ret < 0) {
-		snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(handle->errbuf, PCAP_ERRBUF_SIZE,
 		    "Can't get stats via ioctl: %s", strerror(errno));
 		return (-1);
-		
+
 	}
 
-	/* we receive both rx and tx frames, so comulate all stats */	
-	stats->ps_recv = s->evt_rx + s->acl_rx + s->sco_rx + s->cmd_tx + 
+	/* we receive both rx and tx frames, so comulate all stats */
+	stats->ps_recv = s->evt_rx + s->acl_rx + s->sco_rx + s->cmd_tx +
 		s->acl_tx +s->sco_tx;
 	stats->ps_drop = s->err_rx + s->err_tx;
 	stats->ps_ifdrop = 0;
 	return 0;
 }
 
-static int 
+static int
 bt_setdirection_linux(pcap_t *p, pcap_direction_t d)
 {
 	p->direction = d;
