@@ -24,11 +24,6 @@
  * Packet capture routines for DLPI using libdlpi under SunOS 5.11.
  */
 
-#ifndef lint
-static const char rcsid[] _U_ =
-	"@(#) $Header: /tcpdump/master/libpcap/pcap-libdlpi.c,v 1.6 2008-04-14 20:40:58 guy Exp $ (LBL)";
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -101,26 +96,28 @@ static int
 pcap_activate_libdlpi(pcap_t *p)
 {
 	struct pcap_dlpi *pd = p->priv;
+	int status = 0;
 	int retv;
 	dlpi_handle_t dh;
 	dlpi_info_t dlinfo;
-	int err = PCAP_ERROR;
 
 	/*
 	 * Enable Solaris raw and passive DLPI extensions;
 	 * dlpi_open() will not fail if the underlying link does not support
 	 * passive mode. See dlpi(7P) for details.
 	 */
-	retv = dlpi_open(p->opt.source, &dh, DLPI_RAW|DLPI_PASSIVE);
+	retv = dlpi_open(p->opt.device, &dh, DLPI_RAW|DLPI_PASSIVE);
 	if (retv != DLPI_SUCCESS) {
 		if (retv == DLPI_ELINKNAMEINVAL || retv == DLPI_ENOLINK)
-			err = PCAP_ERROR_NO_SUCH_DEVICE;
+			status = PCAP_ERROR_NO_SUCH_DEVICE;
 		else if (retv == DL_SYSERR &&
 		    (errno == EPERM || errno == EACCES))
-			err = PCAP_ERROR_PERM_DENIED;
-		pcap_libdlpi_err(p->opt.source, "dlpi_open", retv,
+			status = PCAP_ERROR_PERM_DENIED;
+		else
+			status = PCAP_ERROR;
+		pcap_libdlpi_err(p->opt.device, "dlpi_open", retv,
 		    p->errbuf);
-		return (err);
+		return (status);
 	}
 	pd->dlpi_hd = dh;
 
@@ -129,20 +126,21 @@ pcap_activate_libdlpi(pcap_t *p)
 		 * This device exists, but we don't support monitor mode
 		 * any platforms that support DLPI.
 		 */
-		err = PCAP_ERROR_RFMON_NOTSUP;
+		status = PCAP_ERROR_RFMON_NOTSUP;
 		goto bad;
 	}
 
 	/* Bind with DLPI_ANY_SAP. */
 	if ((retv = dlpi_bind(pd->dlpi_hd, DLPI_ANY_SAP, 0)) != DLPI_SUCCESS) {
-		pcap_libdlpi_err(p->opt.source, "dlpi_bind", retv, p->errbuf);
+		status = PCAP_ERROR;
+		pcap_libdlpi_err(p->opt.device, "dlpi_bind", retv, p->errbuf);
 		goto bad;
 	}
 
 	/* Enable promiscuous mode. */
 	if (p->opt.promisc) {
-		err = dlpromiscon(p, DL_PROMISC_PHYS);
-		if (err < 0) {
+		retv = dlpromiscon(p, DL_PROMISC_PHYS);
+		if (retv < 0) {
 			/*
 			 * "You don't have permission to capture on
 			 * this device" and "you don't have permission
@@ -156,57 +154,71 @@ pcap_activate_libdlpi(pcap_t *p)
 			 * XXX - you might have to capture in
 			 * promiscuous mode to see outgoing packets.
 			 */
-			if (err == PCAP_ERROR_PERM_DENIED)
-				err = PCAP_ERROR_PROMISC_PERM_DENIED;
+			if (retv == PCAP_ERROR_PERM_DENIED)
+				status = PCAP_ERROR_PROMISC_PERM_DENIED;
+			else
+				status = retv;
 			goto bad;
 		}
 	} else {
 		/* Try to enable multicast. */
-		err = dlpromiscon(p, DL_PROMISC_MULTI);
-		if (err < 0)
+		retv = dlpromiscon(p, DL_PROMISC_MULTI);
+		if (retv < 0) {
+			status = retv;
 			goto bad;
+		}
 	}
 
 	/* Try to enable SAP promiscuity. */
-	err = dlpromiscon(p, DL_PROMISC_SAP);
-	if (err < 0) {
+	retv = dlpromiscon(p, DL_PROMISC_SAP);
+	if (retv < 0) {
 		/*
 		 * Not fatal, since the DL_PROMISC_PHYS mode worked.
 		 * Report it as a warning, however.
 		 */
 		if (p->opt.promisc)
-			err = PCAP_WARNING;
-		else
+			status = PCAP_WARNING;
+		else {
+			status = retv;
 			goto bad;
+		}
 	}
 
 	/* Determine link type.  */
 	if ((retv = dlpi_info(pd->dlpi_hd, &dlinfo, 0)) != DLPI_SUCCESS) {
-		pcap_libdlpi_err(p->opt.source, "dlpi_info", retv, p->errbuf);
+		status = PCAP_ERROR;
+		pcap_libdlpi_err(p->opt.device, "dlpi_info", retv, p->errbuf);
 		goto bad;
 	}
 
-	if (pcap_process_mactype(p, dlinfo.di_mactype) != 0)
+	if (pcap_process_mactype(p, dlinfo.di_mactype) != 0) {
+		status = PCAP_ERROR;
 		goto bad;
+	}
 
 	p->fd = dlpi_fd(pd->dlpi_hd);
 
 	/* Push and configure bufmod. */
-	if (pcap_conf_bufmod(p, p->snapshot) != 0)
+	if (pcap_conf_bufmod(p, p->snapshot) != 0) {
+		status = PCAP_ERROR;
 		goto bad;
+	}
 
 	/*
 	 * Flush the read side.
 	 */
 	if (ioctl(p->fd, I_FLUSH, FLUSHR) != 0) {
-		snprintf(p->errbuf, PCAP_ERRBUF_SIZE, "FLUSHR: %s",
+		status = PCAP_ERROR;
+		pcap_snprintf(p->errbuf, PCAP_ERRBUF_SIZE, "FLUSHR: %s",
 		    pcap_strerror(errno));
 		goto bad;
 	}
 
 	/* Allocate data buffer. */
-	if (pcap_alloc_databuf(p) != 0)
+	if (pcap_alloc_databuf(p) != 0) {
+		status = PCAP_ERROR;
 		goto bad;
+	}
 
 	/*
 	 * "p->fd" is a FD for a STREAMS device, so "select()" and
@@ -224,10 +236,10 @@ pcap_activate_libdlpi(pcap_t *p)
 	p->stats_op = pcap_stats_dlpi;
 	p->cleanup_op = pcap_cleanup_libdlpi;
 
-	return (0);
+	return (status);
 bad:
 	pcap_cleanup_libdlpi(p);
-	return (err);
+	return (status);
 }
 
 #define STRINGIFY(n)	#n
@@ -246,13 +258,24 @@ dlpromiscon(pcap_t *p, bpf_u_int32 level)
 			err = PCAP_ERROR_PERM_DENIED;
 		else
 			err = PCAP_ERROR;
-		pcap_libdlpi_err(p->opt.source, "dlpi_promiscon" STRINGIFY(level),
+		pcap_libdlpi_err(p->opt.device, "dlpi_promiscon" STRINGIFY(level),
 		    retv, p->errbuf);
 		return (err);
 	}
 	return (0);
 }
 
+/*
+ * Presumably everything returned by dlpi_walk() is a DLPI device,
+ * so there's no work to be done here to check whether name refers
+ * to a DLPI device.
+ */
+static int
+is_dlpi_interface(const char *name _U_)
+{
+	return (1);
+}
+
 /*
  * In Solaris, the "standard" mechanism" i.e SIOCGLIFCONF will only find
  * network links that are plumbed and are up. dlpi_walk(3DLPI) will find
@@ -267,12 +290,18 @@ pcap_platform_finddevs(pcap_if_t **alldevsp, char *errbuf)
 	linkwalk_t	lw = {NULL, 0};
 	int 		save_errno;
 
+	/*
+	 * Get the list of regular interfaces first.
+	 */
+	if (pcap_findalldevs_interfaces(alldevsp, errbuf, is_dlpi_interface) == -1)
+		return (-1);	/* failure */
+
 	/* dlpi_walk() for loopback will be added here. */
 
 	dlpi_walk(list_interfaces, &lw, 0);
 
 	if (lw.lw_err != 0) {
-		snprintf(errbuf, PCAP_ERRBUF_SIZE,
+		pcap_snprintf(errbuf, PCAP_ERRBUF_SIZE,
 		    "dlpi_walk: %s", pcap_strerror(lw.lw_err));
 		retv = -1;
 		goto done;
@@ -325,7 +354,7 @@ pcap_read_libdlpi(pcap_t *p, int count, pcap_handler callback, u_char *user)
 		}
 
 		msglen = p->bufsize;
-		bufp = p->buffer + p->offset;
+		bufp = (u_char *)p->buffer + p->offset;
 
 		retv = dlpi_recv(pd->dlpi_hd, NULL, NULL, bufp,
 		    &msglen, -1, NULL);
@@ -392,16 +421,16 @@ pcap_cleanup_libdlpi(pcap_t *p)
 static void
 pcap_libdlpi_err(const char *linkname, const char *func, int err, char *errbuf)
 {
-	snprintf(errbuf, PCAP_ERRBUF_SIZE, "libpcap: %s failed on %s: %s",
+	pcap_snprintf(errbuf, PCAP_ERRBUF_SIZE, "libpcap: %s failed on %s: %s",
 	    func, linkname, dlpi_strerror(err));
 }
 
 pcap_t *
-pcap_create_interface(const char *device, char *ebuf)
+pcap_create_interface(const char *device _U_, char *ebuf)
 {
 	pcap_t *p;
 
-	p = pcap_create_common(device, ebuf, sizeof (struct pcap_dlpi));
+	p = pcap_create_common(ebuf, sizeof (struct pcap_dlpi));
 	if (p == NULL)
 		return (NULL);
 
