@@ -54,7 +54,7 @@ int* _errno();
 #define errno (*_errno())
 #endif /* __MINGW32__ */
 #ifdef HAVE_REMOTE
-#include "pcap-remote.h"
+#include "pcap-rpcap.h"
 #endif /* HAVE_REMOTE */
 
 static int pcap_setfilter_win32_npf(pcap_t *, struct bpf_program *);
@@ -95,10 +95,10 @@ BOOL WINAPI DllMain(
 
 /*
  * Define stub versions of the monitor-mode support routines if this
- * isn't Npcap.  NPF_DRIVER_NAME_NORMAL is defined by Npcap but not
+ * isn't Npcap. HAVE_NPCAP_PACKET_API is defined by Npcap but not
  * WinPcap.
  */
-#ifndef NPF_DRIVER_NAME_NORMAL
+#ifndef HAVE_NPCAP_PACKET_API
 static int
 PacketIsMonitorModeSupported(PCHAR AdapterName _U_)
 {
@@ -829,6 +829,66 @@ pcap_activate_win32(pcap_t *p)
 	int res;
 	char errbuf[PCAP_ERRBUF_SIZE+1];
 
+#ifdef HAVE_REMOTE
+	char host[PCAP_BUF_SIZE + 1];
+	char port[PCAP_BUF_SIZE + 1];
+	char name[PCAP_BUF_SIZE + 1];
+	int srctype;
+	int opensource_remote_result;
+
+	struct pcap_md *md;				/* structure used when doing a remote live capture */
+	md = (struct pcap_md *) ((u_char*)p->priv + sizeof(struct pcap_win));
+
+	/*
+	Retrofit; we have to make older applications compatible with the remote capture
+	So, we're calling the pcap_open_remote() from here, that is a very dirty thing.
+	Obviously, we cannot exploit all the new features; for instance, we cannot
+	send authentication, we cannot use a UDP data connection, and so on.
+	*/
+	if (pcap_parsesrcstr(p->opt.device, &srctype, host, port, name, p->errbuf))
+		return PCAP_ERROR;
+
+	if (srctype == PCAP_SRC_IFREMOTE)
+	{
+		opensource_remote_result = pcap_opensource_remote(p, NULL);
+
+		if (opensource_remote_result != 0)
+			return opensource_remote_result;
+
+		md->rmt_flags = (p->opt.promisc) ? PCAP_OPENFLAG_PROMISCUOUS : 0;
+
+		return 0;
+	}
+
+	if (srctype == PCAP_SRC_IFLOCAL)
+	{
+		/*
+		* If it starts with rpcap://, cut down the string
+		*/
+		if (strncmp(p->opt.device, PCAP_SRC_IF_STRING, strlen(PCAP_SRC_IF_STRING)) == 0)
+		{
+			size_t len = strlen(p->opt.device) - strlen(PCAP_SRC_IF_STRING) + 1;
+			char *new_string;
+			/*
+			* allocate a new string and free the old one
+			*/
+			if (len > 0)
+			{
+				new_string = (char*)malloc(len);
+				if (new_string != NULL)
+				{
+					char *tmp;
+					strcpy_s(new_string, len, p->opt.device + strlen(PCAP_SRC_IF_STRING));
+					tmp = p->opt.device;
+					p->opt.device = new_string;
+					free(tmp);
+				}
+			}
+		}
+	}
+
+#endif	/* HAVE_REMOTE */
+
 	if (p->opt.rfmon) {
 		/*
 		 * Monitor mode is supported on Windows Vista and later.
@@ -1144,6 +1204,7 @@ pcap_t *
 pcap_create_interface(const char *device _U_, char *ebuf)
 {
 	pcap_t *p;
+
 #ifdef HAVE_REMOTE
 	p = pcap_create_common(ebuf, sizeof(struct pcap_win) + sizeof(struct pcap_md));
 #else
@@ -1152,8 +1213,6 @@ pcap_create_interface(const char *device _U_, char *ebuf)
 	if (p == NULL)
 		return (NULL);
 
-	struct pcap_win *pw = p->priv;
-	pw->rfmon_selfstart = 0;
 	p->activate_op = pcap_activate_win32;
 	p->can_set_rfmon_op = pcap_can_set_rfmon_win32;
 	return (p);
@@ -1449,22 +1508,12 @@ pcap_platform_finddevs(pcap_if_t **alldevsp, char *errbuf)
 	while (*name != '\0') {
 		bpf_u_int32 flags = 0;
 #ifdef HAVE_PACKET_IS_LOOPBACK_ADAPTER
-		pcap_t *p;
-
 		/*
 		 * Is this a loopback interface?
-		 * XXX - it'd be best if there were a way to get a list
-		 * of interfaces *and* flags, including "Up" and "Running"
-		 * as well as "loopback", without having to open the
-		 * interfaces.
 		 */
-		p = pcap_open_live(name, 68, 0, 0, errbuf);
-		if (p != NULL) {
-			if (PacketIsLoopbackAdapter(p->adapter)) {
-				/* Yes */
-				flags |= PCAP_IF_LOOPBACK;
-			}
-			pcap_close(p);
+		if (PacketIsLoopbackAdapter(name)) {
+			/* Yes */
+			flags |= PCAP_IF_LOOPBACK;
 		}
 #endif
 
